import { useMemo, useRef } from 'react';
import { Vector3 } from 'three';
import { useFrame } from '@react-three/fiber';
import { DIRS } from '../utils/constants';
import random from '../utils/random';
import { getCoord } from '../utils/scene';
import vertority from '../utils/vertority';
var DARK_CLOUD_COLORS = ['#21373d', '#535657'];
export var useCloud = function useCloud(cloud) {
  var dir = useRef(1);
  var speed = useRef(Math.random() * 0.001 * random.inRange(DIRS));
  var distance = useRef(0);
  useFrame(function () {
    if (!cloud.current) {
      return;
    }

    distance.current += 0.001 * dir.current;
    cloud.current.position.x += speed.current * dir.current;
    cloud.current.position.y -= speed.current * dir.current;

    if (distance.current <= 0) {
      dir.current = -dir.current;
      speed.current = Math.random() * 0.001 * random.inRange(DIRS);
    } else if (distance.current >= 0.1) {
      dir.current = -dir.current;
      speed.current = Math.random() * 0.001 * random.inRange(DIRS);
    }
  });
};
export var useClouds = function useClouds() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 10,
    colors: DARK_CLOUD_COLORS
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 10 : _ref$count,
      _ref$colors = _ref.colors,
      colors = _ref$colors === void 0 ? DARK_CLOUD_COLORS : _ref$colors;

  var coord = useRef(getCoord()).current;
  var clouds = useMemo(function () {
    return new Array(count).fill(0).map(function () {
      return {
        radius: Math.random() * coord[0] / 2,
        startpoint: vertority.fromAxis('fromTop').add(new Vector3(0, coord[1], 0)),
        opacity: Math.random(),
        color: random.inRange(colors)
      };
    });
  }, [colors, coord, count]);
  return {
    clouds: clouds,
    startpoint: new Vector3(0, coord[1], 0)
  };
};
//# sourceMappingURL=use-cloud.js.map
