import { useRef } from 'react';
import { useFrame } from '@react-three/fiber';
import vertority from '../utils/vertority';
import { computeBoundingbox } from '../utils/element';
import random from '../utils/random';
import { getCoord } from '../utils/scene';
import { DIRS } from '../utils/constants';
import { useRain } from '../rain/use-rain';
import { angle2dir } from '../utils/angle';
var HAZE_COLORS = ['#fff', '#0F203B'];
export var DEFAULT_RAINPROPS = {
  count: 100,
  angle: -45
};
export var useHaze = function useHaze() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_RAINPROPS,
      _ref$angle = _ref.angle,
      angle = _ref$angle === void 0 ? -45 : _ref$angle,
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 100 : _ref$count;

  var _useRain = useRain({
    angle: angle,
    count: count,
    colors: HAZE_COLORS
  }),
      lines = _useRain.lines;

  return {
    lines: lines.map(function (line) {
      return Object.assign(Object.assign({}, line), {
        opacity: Math.random() + 0.5,
        dashArray: Math.random() * 0.3
      });
    })
  };
};
export var useHazeDrop = function useHazeDrop(hazedrop, mat, props) {
  var friction = useRef(random.inRange(DIRS) === 1 ? 0 : 1 * 0.01 * Math.round(Math.random() * 10)).current;
  var coord = useRef(getCoord()).current;
  var vy0 = useRef(0.001 + friction); // vy0 / vx0 = tan(angle)

  var vx0 = useRef(0.001 + friction);
  var a = useRef(0.002);

  var _computeBoundingbox = computeBoundingbox(props === null || props === void 0 ? void 0 : props.value.vertices[0]),
      offsetTop = _computeBoundingbox.offsetTop;

  useFrame(function () {
    var _a;

    if (((_a = hazedrop.current) === null || _a === void 0 ? void 0 : _a.position.y) === undefined || !(props === null || props === void 0 ? void 0 : props.value) || !mat.current) {
      return;
    } // hazedrop加速下落


    hazedrop.current.position.y -= vy0.current; // left hazedrop从左到右移动, right hazedrop从右到左移动

    hazedrop.current.position.x -= vx0.current * angle2dir(props.value.angle); // 从无到有比较真实

    vy0.current += a.current;
    vx0.current += a.current; // 判断hazedrop是否出了边界

    if (offsetTop + Math.abs(hazedrop.current.position.y) > coord[1] * 2 + props.value.leg) {
      var vertor = vertority.fromPlacement(props.value.orientation); // 随机hazedrop初始位置, 避免loop重复

      hazedrop.current.position.set(vertor.x, vertor.y, vertor.z);
      vy0.current = 0.0001;
      vx0.current = 0.0001;
    }
  });
};
//# sourceMappingURL=use-haze.js.map
