import { useMemo, useRef } from 'react';
import { Vector3 } from 'three';
import { useFrame } from '@react-three/fiber';
import vertority from '../utils/vertority';
import { computeBoundingbox } from '../utils/element';
import { deg2rad } from '../utils/angle';
import { getCoord } from '../utils/scene';
export var useMeteors = function useMeteors() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 10,
    angle: 30
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 5 : _ref$count,
      _ref$angle = _ref.angle,
      angle = _ref$angle === void 0 ? 30 : _ref$angle;

  var meteors = useMemo(function () {
    var _angle = deg2rad(angle);

    var coord = getCoord();
    return new Array(count).fill(0).map(function () {
      var leg = Math.random() * coord[1]; // noise vertor for startpoint

      var vertor = vertority.fromPlacement('fromRight');
      return {
        vertices: [// endpoint
        new Vector3().copy(vertor) // distance vertor
        .add(new Vector3(leg / Math.tan(_angle), leg, 0)), // startpoint
        new Vector3().copy(vertor)],
        angle: _angle,
        leg: leg,
        hypotenuse: leg / Math.sin(_angle),
        color: 'white'
      };
    });
  }, [count, angle]);
  return {
    meteors: meteors
  };
};
export var useMeteor = function useMeteor(meteor, mat, _ref2) {
  var value = _ref2.value;
  var vopacity = useRef(0.01);

  var _computeBoundingbox = computeBoundingbox(value.vertices[0]),
      offsetTop = _computeBoundingbox.offsetTop;

  var threshold = Math.random() * 8;
  useFrame(function () {
    var _a;

    if (((_a = meteor.current) === null || _a === void 0 ? void 0 : _a.position.y) === undefined || !mat.current) {
      return;
    } // 判断meteor是否出了边界


    if (offsetTop + Math.abs(meteor.current.position.y) > threshold) {
      mat.current.uniforms.dashOffset.value -= Math.abs(vopacity.current);
      mat.current.opacity -= vopacity.current;
    } else {
      // meteor加速下落
      meteor.current.position.y -= Math.sin(value.angle) * Math.abs(vopacity.current) * 10;
      meteor.current.position.x -= Math.cos(value.angle) * Math.abs(vopacity.current) * 10;
    }

    if (Math.abs(mat.current.uniforms.dashOffset.value) >= 1.1) {
      mat.current.opacity = 1;
      mat.current.uniforms.dashOffset.value = 0;
      var vertor = vertority.fromAxis('fromRight'); // 随机meteor初始位置, 避免loop重复

      meteor.current.position.set(vertor.x, vertor.y, vertor.z);
    }
  });
};
//# sourceMappingURL=use-meteors.js.map
