import { useMemo, useRef } from 'react';
import * as THREE from 'three';
import random from '../utils/random';
import { DIRS } from '../utils/constants';
import { useFrame } from '@react-three/fiber';
import { getCoord } from '../utils/scene';
var RING_COLORS = ['#cdd1d3', '#fcd337', '#1677b3'];
export var useStarRings = function useStarRings() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 50
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 50 : _ref$count;

  var coord = useRef(getCoord()).current;
  var startpoint = useMemo(function () {
    return new THREE.Vector3(-coord[0], coord[1], 0);
  }, [coord]);
  var rings = useMemo(function () {
    return new Array(count).fill(0).map(function () {
      var radius = Math.random() * 10;
      var vertices = new Array(180).fill(0).map(function (_v, i) {
        return new THREE.Vector3(Math.cos(i * 2 * Math.PI / 180) * radius, Math.sin(i * 2 * Math.PI / 180) * radius, 0);
      });
      return {
        radius: radius,
        vertices: vertices,
        dashArray: Math.random() + 0.1,
        opacity: Math.random() * 0.8,
        color: random.inRange(RING_COLORS),
        lineWidth: Math.random() * 0.05
      };
    });
  }, [count]);
  return {
    rings: rings,
    startpoint: startpoint
  };
};
export var useRing = function useRing(ring) {
  var dir = random.inRange(DIRS);
  var speed = useRef(Math.random() * 2 * 0.0001);
  useFrame(function () {
    if (!ring.current) {
      return;
    }

    ring.current.uniforms.dashOffset.value -= speed.current * dir;
  });
};
//# sourceMappingURL=use-starrings.js.map
