"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useClouds = exports.useCloud = void 0;

var _react = require("react");

var _three = require("three");

var _fiber = require("@react-three/fiber");

var _constants = require("../utils/constants");

var _random = _interopRequireDefault(require("../utils/random"));

var _scene = require("../utils/scene");

var _vertority = _interopRequireDefault(require("../utils/vertority"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DARK_CLOUD_COLORS = ['#21373d', '#535657'];

var useCloud = function useCloud(cloud) {
  var dir = (0, _react.useRef)(1);
  var speed = (0, _react.useRef)(Math.random() * 0.001 * _random.default.inRange(_constants.DIRS));
  var distance = (0, _react.useRef)(0);
  (0, _fiber.useFrame)(function () {
    if (!cloud.current) {
      return;
    }

    distance.current += 0.001 * dir.current;
    cloud.current.position.x += speed.current * dir.current;
    cloud.current.position.y -= speed.current * dir.current;

    if (distance.current <= 0) {
      dir.current = -dir.current;
      speed.current = Math.random() * 0.001 * _random.default.inRange(_constants.DIRS);
    } else if (distance.current >= 0.1) {
      dir.current = -dir.current;
      speed.current = Math.random() * 0.001 * _random.default.inRange(_constants.DIRS);
    }
  });
};

exports.useCloud = useCloud;

var useClouds = function useClouds() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 10,
    colors: DARK_CLOUD_COLORS
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 10 : _ref$count,
      _ref$colors = _ref.colors,
      colors = _ref$colors === void 0 ? DARK_CLOUD_COLORS : _ref$colors;

  var coord = (0, _react.useRef)((0, _scene.getCoord)()).current;
  var clouds = (0, _react.useMemo)(function () {
    return new Array(count).fill(0).map(function () {
      return {
        radius: Math.random() * coord[0] / 2,
        startpoint: _vertority.default.fromAxis('fromTop').add(new _three.Vector3(0, coord[1], 0)),
        opacity: Math.random(),
        color: _random.default.inRange(colors)
      };
    });
  }, [colors, coord, count]);
  return {
    clouds: clouds,
    startpoint: new _three.Vector3(0, coord[1], 0)
  };
};

exports.useClouds = useClouds;
//# sourceMappingURL=use-cloud.js.map
