"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useMeteor = exports.useMeteors = void 0;

var _react = require("react");

var _three = require("three");

var _fiber = require("@react-three/fiber");

var _vertority = _interopRequireDefault(require("../utils/vertority"));

var _element = require("../utils/element");

var _angle2 = require("../utils/angle");

var _scene = require("../utils/scene");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var useMeteors = function useMeteors() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 10,
    angle: 30
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 5 : _ref$count,
      _ref$angle = _ref.angle,
      angle = _ref$angle === void 0 ? 30 : _ref$angle;

  var meteors = (0, _react.useMemo)(function () {
    var _angle = (0, _angle2.deg2rad)(angle);

    var coord = (0, _scene.getCoord)();
    return new Array(count).fill(0).map(function () {
      var leg = Math.random() * coord[1]; // noise vertor for startpoint

      var vertor = _vertority.default.fromPlacement('fromRight');

      return {
        vertices: [// endpoint
        new _three.Vector3().copy(vertor) // distance vertor
        .add(new _three.Vector3(leg / Math.tan(_angle), leg, 0)), // startpoint
        new _three.Vector3().copy(vertor)],
        angle: _angle,
        leg: leg,
        hypotenuse: leg / Math.sin(_angle),
        color: 'white'
      };
    });
  }, [count, angle]);
  return {
    meteors: meteors
  };
};

exports.useMeteors = useMeteors;

var useMeteor = function useMeteor(meteor, mat, _ref2) {
  var value = _ref2.value;
  var vopacity = (0, _react.useRef)(0.01);

  var _computeBoundingbox = (0, _element.computeBoundingbox)(value.vertices[0]),
      offsetTop = _computeBoundingbox.offsetTop;

  var threshold = Math.random() * 8;
  (0, _fiber.useFrame)(function () {
    var _a;

    if (((_a = meteor.current) === null || _a === void 0 ? void 0 : _a.position.y) === undefined || !mat.current) {
      return;
    } // 判断meteor是否出了边界


    if (offsetTop + Math.abs(meteor.current.position.y) > threshold) {
      mat.current.uniforms.dashOffset.value -= Math.abs(vopacity.current);
      mat.current.opacity -= vopacity.current;
    } else {
      // meteor加速下落
      meteor.current.position.y -= Math.sin(value.angle) * Math.abs(vopacity.current) * 10;
      meteor.current.position.x -= Math.cos(value.angle) * Math.abs(vopacity.current) * 10;
    }

    if (Math.abs(mat.current.uniforms.dashOffset.value) >= 1.1) {
      mat.current.opacity = 1;
      mat.current.uniforms.dashOffset.value = 0;

      var vertor = _vertority.default.fromAxis('fromRight'); // 随机meteor初始位置, 避免loop重复


      meteor.current.position.set(vertor.x, vertor.y, vertor.z);
    }
  });
};

exports.useMeteor = useMeteor;
//# sourceMappingURL=use-meteors.js.map
