"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useRainRings = exports.useRainRing = void 0;

var _fiber = require("@react-three/fiber");

var _react = require("react");

var _vertority = _interopRequireDefault(require("../utils/vertority"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var useRainRing = function useRainRing(rainring, props) {
  var _useThree = (0, _fiber.useThree)(),
      camera = _useThree.camera;

  (0, _fiber.useFrame)(function () {
    var _a, _b;

    if (!rainring.current || !rainring.current.material) {
      return;
    }

    var mat = rainring.current.material;

    if (camera.rotation.z >= 0 || camera.rotation.y <= 0) {
      mat.opacity = 0;
      return;
    }

    mat.opacity -= 0.01;
    mat.opacity *= (_b = (_a = props.style) === null || _a === void 0 ? void 0 : _a.opacity.get()) !== null && _b !== void 0 ? _b : 1;
    rainring.current.scale.x += 0.1;
    rainring.current.scale.y += 0.1;

    if (rainring.current.material.opacity <= 0) {
      var vertor = _vertority.default.random();

      rainring.current.position.set(vertor.x, vertor.y, vertor.z);
      mat.opacity = 0.2;
      rainring.current.scale.x = 0;
      rainring.current.scale.y = 0;
    }

    rainring.current.rotation.x = camera.rotation.x;
    rainring.current.rotation.y = camera.rotation.y;
    rainring.current.rotation.z = camera.rotation.z;
  });
};

exports.useRainRing = useRainRing;

var useRainRings = function useRainRings() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 10
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 10 : _ref$count;

  var rings = (0, _react.useMemo)(function () {
    return new Array(count).fill(0).map(function () {
      return {
        radius: Math.random() * 0.1,
        startpoint: _vertority.default.random()
      };
    });
  }, [count]);
  return {
    rings: rings
  };
};

exports.useRainRings = useRainRings;
//# sourceMappingURL=use-rain-ring.js.map
