"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useRaindrop = exports.useRain = exports.DEFAULT_RAINPROPS = void 0;

var _fiber = require("@react-three/fiber");

var THREE = _interopRequireWildcard(require("three"));

var _react = require("react");

var _element = require("../utils/element");

var _random = _interopRequireDefault(require("../utils/random"));

var _vertority = _interopRequireDefault(require("../utils/vertority"));

var _point = _interopRequireDefault(require("../utils/point"));

var _angle2 = require("../utils/angle");

var _constants = require("../utils/constants");

var _scene = require("../utils/scene");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var RAIN_COLORS = ['#cdd1d3', '#fcd337'];

var angle2placement = function angle2placement(angle) {
  if (!angle || angle === 0) {
    return 'fromTop';
  }

  return angle < 0 ? 'fromRight' : 'fromLeft';
};

var DEFAULT_RAINPROPS = {
  count: 100,
  angle: -45
};
exports.DEFAULT_RAINPROPS = DEFAULT_RAINPROPS;

var useRain = function useRain() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_RAINPROPS,
      _ref$angle = _ref.angle,
      angle = _ref$angle === void 0 ? -45 : _ref$angle,
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 100 : _ref$count,
      _ref$colors = _ref.colors,
      colors = _ref$colors === void 0 ? RAIN_COLORS : _ref$colors;

  // raindrop start position data
  var _angle = (0, _angle2.deg2rad)(angle);

  var startpoints = (0, _react.useRef)({
    fromTop: new THREE.Vector3().fromArray(_point.default.axisxy.top),
    fromRight: new THREE.Vector3().fromArray(_point.default.axisxy.right),
    fromLeft: new THREE.Vector3().fromArray(_point.default.axisxy.left)
  }).current; // radindrop come from which orientation

  var comefrom = (0, _react.useRef)({
    fromLeft: ['fromTop', 'fromRight'],
    fromRight: ['fromTop', 'fromLeft'],
    fromTop: ['fromTop']
  }).current;
  var lines = (0, _react.useMemo)(function () {
    return Array(count).fill(0).map(function () {
      // h / deltax = tan(ang)
      // 直角边
      var leg = Math.random() * 2; // FIXME: should modify from angle

      var orientation = _random.default.inRange(comefrom[angle2placement(_angle)]); // noise vertor for startpoint


      var vertor = _vertority.default.fromPlacement(orientation).add(new THREE.Vector3(0, 0, _point.default.fromAxisZ()));

      return {
        vertices: [// endpoint
        new THREE.Vector3().copy(startpoints[orientation]).add(vertor) // distance vertor
        .add(new THREE.Vector3(leg / Math.tan(_angle), leg, 0)), // startpoint
        new THREE.Vector3().copy(startpoints[orientation]).add(vertor)],
        angle: _angle,
        leg: leg,
        orientation: orientation,
        color: _random.default.inRange(colors)
      };
    });
  }, [_angle, comefrom, count, startpoints, colors]);
  return {
    lines: lines
  };
};

exports.useRain = useRain;

var useRaindrop = function useRaindrop(raindrop, mat, props) {
  // 摩擦系数, raindrop从负数开始运动, 更加随机的效果
  var friction = (0, _react.useRef)(_random.default.inRange(_constants.DIRS) === 1 ? 0 : -1 * 0.01 * Math.round(Math.random() * 10)).current;
  var coord = (0, _react.useRef)((0, _scene.getCoord)()).current;
  var vy0 = (0, _react.useRef)(0.0001 + friction); // vy0 / vx0 = tan(angle)

  var vx0 = (0, _react.useRef)(0.0001 + friction);
  var a = (0, _react.useRef)(0.001);

  var _computeBoundingbox = (0, _element.computeBoundingbox)(props === null || props === void 0 ? void 0 : props.value.vertices[0]),
      offsetTop = _computeBoundingbox.offsetTop;

  (0, _fiber.useFrame)(function () {
    var _a, _b, _c;

    if (((_a = raindrop.current) === null || _a === void 0 ? void 0 : _a.position.y) === undefined || !(props === null || props === void 0 ? void 0 : props.value) || !mat.current) {
      return;
    } // raindrop加速下落


    raindrop.current.position.y -= vy0.current; // left raindrop从左到右移动, right raindrop从右到左移动

    raindrop.current.position.x -= vx0.current * (0, _angle2.angle2dir)(props.value.angle); // 从无到有比较真实

    mat.current.opacity += 0.01;
    mat.current.opacity *= (_c = (_b = props.style) === null || _b === void 0 ? void 0 : _b.opacity.get()) !== null && _c !== void 0 ? _c : 1;
    vy0.current += a.current;
    vx0.current += a.current; // 判断raindrop是否出了边界

    if (offsetTop + Math.abs(raindrop.current.position.y) > coord[1] * 2 + props.value.leg) {
      var vertor = _vertority.default.fromPlacement(props.value.orientation); // 随机raindrop初始位置, 避免loop重复


      raindrop.current.position.set(vertor.x, vertor.y, vertor.z);
      mat.current.opacity = 0;
      vy0.current = 0.0001;
      vx0.current = 0.0001;
    }
  });
};

exports.useRaindrop = useRaindrop;
//# sourceMappingURL=use-rain.js.map
