"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSnowflake = exports.useSnowflakes = void 0;

var _react = require("react");

var _three = require("three");

var _fiber = require("@react-three/fiber");

var _constants = require("../utils/constants");

var _random = _interopRequireDefault(require("../utils/random"));

var _element = require("../utils/element");

var _scene = require("../utils/scene");

var _vertority = _interopRequireDefault(require("../utils/vertority"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var useSnowflakes = function useSnowflakes() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    count: 100
  },
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 100 : _ref$count;

  var snowflakes = (0, _react.useMemo)(function () {
    var coord = (0, _scene.getCoord)();
    return new Array(4).fill(0).reduce(function (prev, _cur, i) {
      return prev.concat(new Array(Math.round(count / 4)).fill(0).map(function () {
        var vertor = _vertority.default.fromPlacement('fromTop');

        return {
          startpoint: new _three.Vector3().copy(new _three.Vector3(0, coord[1] + i * 2, 0)).add(vertor),
          radius: Math.random() * 0.1
        };
      }));
    }, []);
  }, [count]);
  return {
    snowflakes: snowflakes
  };
};

exports.useSnowflakes = useSnowflakes;

var useSnowflake = function useSnowflake(flake, _ref2) {
  var value = _ref2.value;
  var vy0 = (0, _react.useRef)(0.01);
  var coord = (0, _react.useRef)((0, _scene.getCoord)()).current; // vy0 / vx0 = tan(angle)

  var vx0 = (0, _react.useRef)(0.001 * Math.random() * _random.default.inRange(_constants.DIRS)); // const a = useRef(0.00001)

  var _computeBoundingbox = (0, _element.computeBoundingbox)(value.startpoint),
      offsetTop = _computeBoundingbox.offsetTop;

  (0, _fiber.useFrame)(function () {
    if (!flake.current) {
      return;
    } // 雪花加速下落


    flake.current.position.y -= vy0.current;
    flake.current.position.x -= vx0.current; // 判断是否出了边界

    if (offsetTop + Math.abs(value.startpoint.y - flake.current.position.y) > coord[1] * 2) {
      var vertor = _vertority.default.fromAxis('fromTop'); // 随机raindrop初始位置, 避免loop重复


      flake.current.position.set(vertor.x, vertor.y + coord[1], vertor.z);
      vy0.current = 0.01;
      vx0.current = 0.001 * Math.random() * _random.default.inRange(_constants.DIRS);
    }
  });
};

exports.useSnowflake = useSnowflake;
//# sourceMappingURL=use-snowflake.js.map
